package model

import (
	"github.com/astaxie/beego/orm"
	"time"
)

type Whitelist struct {
	ID            int64     `orm:"column(id)" db:"id" json:"id" form:"id"`
	ProfitAddress string    `orm:"column(profit_address)" db:"profit_address" json:"profit_address" form:"profit_address"`
	NodeNum       int64     `orm:"column(node_num)" db:"node_num" json:"node_num" form:"node_num"`
	CreatedTime   time.Time `orm:"column(created_time)" db:"created_time" json:"created_time" form:"created_time"`
	UpdatedTime   time.Time `orm:"column(updated_time)" db:"updated_time" json:"updated_time" form:"updated_time"`
	Deleted       int64     `orm:"column(deleted)" db:"deleted" json:"deleted" form:"deleted"`
}

func (Whitelist) TableName() string {
	return "whitelist"
}

type WhiteListRepository interface {
	Create(wh *Whitelist) error
	GetByCharacter(addr string) (*Whitelist, error)
	GetListByFilter(filters ...interface{}) []*Whitelist
	Update(wh *Whitelist) error
	Delete(wh *Whitelist) error
}

type whiteListRepositoryImpl struct {
	o orm.Ormer
}

func NewWhiteListRepository() WhiteListRepository {
	o := orm.NewOrm()
	return &whiteListRepositoryImpl{o}
}

func (repo *whiteListRepositoryImpl) Create(wh *Whitelist) error {
	_, err := repo.o.Insert(wh)
	return err
}

func (repo *whiteListRepositoryImpl) GetListByFilter(filters ...interface{}) []*Whitelist {
	list := make([]*Whitelist, 0)
	query := orm.NewOrm().QueryTable(new(Whitelist).TableName())
	if len(filters) > 0 {
		l := len(filters)
		for k := 0; k < l; k += 2 {
			query = query.Filter(filters[k].(string), filters[k+1])
		}
	}
	query.OrderBy("-id").All(&list)
	return list
}

//// return balance, charge, credits and error.
//func (repo *userRepositoryImpl) GetPayInfoById(uid int64) (int64, int64, int64, error) {
//
//}

func (repo *whiteListRepositoryImpl) GetByCharacter(addr string) (*Whitelist, error) {
	wh := Whitelist{ProfitAddress: addr}
	err := repo.o.Read(&wh, "profit_address")
	if err != nil {
		return nil, err
	}
	return &wh, nil
}

func (repo *whiteListRepositoryImpl) Update(wh *Whitelist) error {
	_, err := repo.o.Update(wh)
	return err
}

func (repo *whiteListRepositoryImpl) Delete(wh *Whitelist) error {
	_, err := repo.o.Delete(wh)
	return err
}
